/* Public domain code by Sam Trenholme 2000 */
/* Routines in the string library not dependent on the underlying OS */

/* Headers for the string routines */
#include "JessStr.h"

/* js_init: Create a new string object
   input: The maximum unit count and unit size of the new string object
   output: A pointer to the new string object */

JessicaString *js_create(int max_count, int unit_size) {
    JessicaString *new;

    if((new = js_alloc(1,sizeof(JessicaString))) == -1) 
        return -1;

    /* Initial values for the new string object */
    new->unit_size = unit_size;
    new->unit_count = 0;
    new->max_count = max_count;
    new->is_good = SANE_NUMBER; 
    /* Allocate memory for character string, return on error */
    if((new->string = js_alloc(max_count,unit_size)) == -1) {
        js_dealloc(new);
        return -1;
        }

    return new;
    }

/* js_destroy: Destroy a new string object
   input: Pointer to the string object to destroy
   output: (to do) -1 on error */

int js_destroy(JessicaString *object) {
    /* Deallocate the character string in the string object */
    js_delloc(object->string);
    /* Deallocate the object itself */
    js_delloc(object);
    /* Bye bye */
    return 1;
    }

/* js_has_sanity: Make sure a JessicaString object is sane
   input: pointer to JessicaString object
   output: -1 if insane, otherwise 1
*/

int js_has_sanity(JessicaString *object) {
    /* Is the sane value sane? */
    if(object->is_good != SANE_NUMBER)
        return -1;

    /* Is the length not greater than the max length? */
    if(object->unit_count > object->max_count)
        return -1;

    return 1;
    }


/* js_str2js: Convert a 'normal' C-lang string to a JessicaString object
   input: pointer to js object, string to convert, units to convert, size
          of each unit in octets
   output: -1 on error */

int js_str2js(JessicaString *js, char *string, int count, int size) {
    int counter = 0;
    int max = (count > js->max_count ? js->max_count : count);

    /* Sanity checks */
    if(js_has_sanity(js) == -1)
        return -1;
    if(size != js->unit_size)
        return -1;

    while(counter < (max * size)) {
        *(js->string + counter) = *(string + counter);
	counter++;
        }

    return 1;
    }

/* js_issame: Determine if two JessicaString objects are identical
   input: Pointers to the two string objects
   output: 1 if they are the same, 0 otherwise, -1 on error */

int js_issame(JessicaString *js1, JessicaString *js2) {
    int counter = 0;
    int max;

    /* Sanity checks */
    if(js_has_sanity(js1) == -1)
        return -1;
    if(js_has_sanity(js2) == -1)
        return -1;

    /* They are not the same if they have different sizes for a character */
    if(js1->unit_size != js2->unit_size)
        return 0;
    /* Nor are they the same if they have different lengths */
    if(js1->unit_count != js2->unit_count)
        return 0;

    max = js1->unit_count;

    /* If any characters in the actual string differer, they are different */
    while(counter < (max * js1->unit_size)) {
        if(*(js1->string + counter) != *(js2->string + counter)) 
            return 0; 
        counter++;
	}
   
    /* Otherwise, they are identical */
    return 1;
    }

/* js_fgrep: Determine if the contents of one JessicaString object is
             embedded in another JessicaString object
   input: Object with string to look for, string object to look in
   output: number of units from beginning of string grepped string is
           at if found, 0 otherwise, -1 on error */

int js_fgrep(JessicaString *exp, JessicaString *js) {
    int counter = 0;
    int place = 0;
    int match = 0;
    int max;

    /* Sanity checks */
    if(js_has_sanity(exp) == -1)
        return -1;
    if(js_has_sanity(js) == -1)
        return -1;

    /* They are not the same if they have different sizes for a character */
    if(exp->unit_size != js->unit_size)
        return 0;

    /* They do not match if the grep expression is longer than the string
       to grep from */
    if(exp->unit_count > js->unit_count)
        return 0;

    /* In the case of an empty grep expression, they match (at the top) */
    if(exp->unit_count <= 0)
        return 1;

    /* Max here is the last letter a matching expression can start at
       in the string we are looking for the expression in */
    max = js->unit_count - exp->unit_count;

    /* Here is the actual grepping core */
    while(counter <= (max * js->unit_size)) {
        /* If we match at the beginning of a letter */
        if(*(exp->string + counter) == *(js->string + counter) &&
          counter % js->unit_size == 0) {
	    /* We make sure we match for each letter in the expression */
	    match = counter;
	    place = 0;
	    while(place < (exp->unit_count * exp->unit_size) &&
	      place + counter < (js->unit_count * js->unit_size)) {
	        if(*(exp->string + place) != 
		  *(js->string + place + counter))
		    match = 0;
                place++;
		}
            }
        counter++;
	}
   
    /* Return whether they matched or not */
    return match;

    }

